#!/usr/bin/python
# encoding: utf-8

import sys

DEFAULT_COLUMN_WIDTH = 15
DEFAULT_COLUMN_SEP = "|"
INF = float("+inf")

class Table:
    def __init__(self, dest = sys.stdout, column_width = DEFAULT_COLUMN_WIDTH, column_sep = DEFAULT_COLUMN_SEP):
        self.dest = dest
        self.column_width = column_width
        self.column_sep = column_sep

    def write_column(self, *values):
        # num of seps for sub-columns
        seps = len(values) - 1
        sub_column_width = float(self.column_width - len(self.column_sep) * seps) / len(values)
        column_num = 1
        width_sum = 0
        column = ""
        for value in values:
            width = int(column_num * sub_column_width - width_sum)
            column += value.center(width) + self.column_sep
            column_num += 1
            width_sum += width
        self.dest.write(column)
        self.dest.flush()
    
    def write_columns(self, columns):
        for column in columns:
            if type(column) == list:
                self.write_column(*column)
            else:
                self.write_column(column)
    
    def end_row(self):
        self.dest.write('\n')

def get_instance_name(instance_file):
    import os
    instance_name, ext = os.path.splitext(os.path.basename(instance_file))
    return instance_name
 
def exec_prog_with_config(prog, config, instance_file):
    from time import time
    from subprocess import check_output

    start = time()
    r = check_output([prog, config, instance_file]).strip()
    elapsed = time() - start

    if not r:
        print
        print u"Nenhum resultado gerado pela execução do comando:"
        print " ".join([prog, config, instance_file])
        sys.exit(1)
    
    space = r.index(" ")
    cost = float(r[0:space])
    return elapsed, cost, r[space+1:]

def exec_prog_with_configs(prog, configs, instance_file, table):
    table.write_column(get_instance_name(instance_file))
    best_cost = INF
    for config in configs:
        time, cost, solution = exec_prog_with_config(prog, config, instance_file)
        if cost < best_cost:
            best_cost = cost
            best_config = config
        table.write_column("%.2f" % (time), "%.0f" % (cost))
    table.write_column(best_config)
    table.end_row()

def show_usage():
    #TODO: explicar melhor a forma de uso
    print u"Poucos parâmetros"
    print u"Modo de usar: %s nome-do-programa configurações lista-arquivos" % (sys.argv[0])

def parse_params():
    if len(sys.argv) < 4:
        show_usage()
        sys.exit(1)

    prog = sys.argv[1]
    configs = sys.argv[2].split(",")
    files = sys.argv[3:]

    return prog, configs, files

def main():
    prog, configs, files = parse_params()

    table = Table()
    table.write_columns([""] + configs + ["Melhor"])
    table.end_row()
    table.write_columns([u"Instância"] + [["T", "S"]] * len(configs) + [u"Configuração"])
    table.end_row()

    for instance_file in files:
        exec_prog_with_configs(prog, configs, instance_file, table)

if __name__ == "__main__":
    main()
